#=======================================================================
# this is where everything that is site-specific gets defined
#=======================================================================

package AbsenceConfig;

use FileHandle;

BEGIN {

	$VERSION	= '1.3';
	$DEBUG		= 0;

	#-------------------------------------------------------------
	# If you want multiple instances of absence, you MUST add
	# an instance-name and corresponding path to the config-file
	# below.  Do not include the config-file in the path.  The
	# config-file is ALWAYS called "absence.conf".
	#
	# Additionally you must edit your top-html file (the one which
	# defines the frames) and add "?instance=<your-instance>" to
	# the end of both frame target-URLs.  For example:
	#
	#	SRC="/cgi-bin/absence-control.pl?instance=foo"
	#	SRC="/cgi-bin/absence.pl?instance=foo"
	#
	# for an instance you call "foo"
	#-------------------------------------------------------------
	%INSTANCES = (
		# example
		#foo	=> '/home/httpd/urban/www.unix-wissen.de/html/foo',
	);

	#-------------------------------------------------------------
	# DATA and CGI directories
	# ACHTUNG.  You will probably want to set these
	#-------------------------------------------------------------

	# the following variables are declared for convenience

	#-------------------------------------------------------------
	# this is the relative path to the data-dir.
	#-------------------------------------------------------------
	$data_dir_rel	= '/absence';					# as used in a webpage

	#-------------------------------------------------------------
	# This is the absolute path to the data directory, for the
	# use of a perl script looking for data-files.
	#
	# users of windoze/IIS will probably need to shove some
	# absolute path into "$data_dir_abs". For example:
	#	$data_dir_abs = "c:/inetpub/wwwroot$data_dir_rel";
	#-------------------------------------------------------------
	$data_dir_abs	= "$ENV{DOCUMENT_ROOT}$data_dir_rel";	# as used by Perl

	#-------------------------------------------------------------
	# the relative path to the cgi-bin, i.e., what is needed to
	# create a URL for a cgi-script.
	#-------------------------------------------------------------
	$cgi_dir_rel	= "/cgi-bin";

	%VARS = (
		data_dir_rel	=> $data_dir_rel,
		data_dir_abs	=> $data_dir_abs,
		cgi_dir_rel		=> $cgi_dir_rel,

		#-------------------------------------------------------------
		# where are the images kept.  Probably OK as is.
		#-------------------------------------------------------------

		image_dir_rel	=> "$data_dir_rel/img",
		image_dir_abs	=> "$data_dir_abs/img",

		#-------------------------------------------------------------
		# where is absence database.  Probably OK as is.
		#-------------------------------------------------------------

		database_file	=> "$data_dir_abs/absence.db",

		#-------------------------------------------------------------
		# where is modification time db.  Probably OK as is.
		#-------------------------------------------------------------

		modtime_db_file	=> "$data_dir_abs/month_mod_times.db",

		#-------------------------------------------------------------
		# where is holiday.db file
		#-------------------------------------------------------------

		holiday_db_file	=> "$data_dir_abs/holiday.db",

		#-------------------------------------------------------------
		# where is log file.  Probably OK as is.
		#-------------------------------------------------------------

		log_file		=> "$data_dir_abs/log",

		#-------------------------------------------------------------
		# where is type file.  Probably OK as is.
		#-------------------------------------------------------------

		type_file		=> "$data_dir_abs/absence-types.db",

		#-------------------------------------------------------------
		# paths to various CGI-scripts
		#-------------------------------------------------------------

		main_script		=> "$cgi_dir_rel/absence.pl",
		manage_script	=> "$cgi_dir_rel/absence-manage.pl",
		control_script	=> "$cgi_dir_rel/absence-control.pl",

		#-------------------------------------------------------------
		# legend-style.  can either be "constant:<num>" or "fit"
		# if set to constant, all legend-labels will be <num> pixels wide
		# if "fit" labels will be fit to text
		#-------------------------------------------------------------

		#legend_style	=> 'constant:60',
		legend_style	=> 'fit',

		#-------------------------------------------------------------
		# the number of backup copies of the dababase file.
		# a new copy is made each time the database is modified
		#-------------------------------------------------------------

		backup_copies	=> 10,

		#-------------------------------------------------------------
		# select the image-type, png, jpg, or gif.  'gif' is only
		# supported on ancient versions of gd.  'png' is probably
		# the best choice if you have it.
		#-------------------------------------------------------------
		image_type		=> 'png',	# one of 'png', 'jpg', or 'gif'

		#-------------------------------------------------------------
		# map_type determines whether client or server-side image-maps
		# are used.  The value 'client' causes a map to be generated
		# and integrated into the main web-page for each month.  This
		# can cause thousands of map-areas to be defined, depending
		# on how many months are displayed and how many people are
		# in a group.
		# The value 'server' causes the "ismap" option to be added to
		# the month-image, which additionally is a hyperlink. This
		# results in the coordinates of the click to be passed to
		# the server for processing.
		# The advantage of the client-side option is tooltips are used
		# to display the contents of a reservation.
		#
		# legal values: "client" or "server"
		#-------------------------------------------------------------
		map_type		=> 'client',
		#map_type		=> 'server',

		#-------------------------------------------------------------
		# you will want to set this to your domain, otherwise cookies
		# won't work.
		#-------------------------------------------------------------
		cookie_domain	=> '.unix-wissen.de',
	);
}

if (exists($ENV{INSTANCE_NAME})) {
	# an instance has been specified
	my $inst = $ENV{INSTANCE_NAME};
	print "** instance specified: [$inst] **<BR>\n";
	if (exists($INSTANCES{$inst})) {
		my $cf = "$INSTANCES{$inst}/absence.conf";
		if (! -e $cf) {
			die "no configfile found for instance [$inst] ($cf)\n";
		}
		parse($cf);
	} else {
		die "instance [$inst] not defined\n";
	}
}

sub fetch
{
	my $val = shift;

	if (exists($VARS{$val})) {
		return $VARS{$val};
	}

	return undef;
}

sub parse
{
	my ($file) = @_;

	my $fh = FileHandle->new($file);
	defined($fh) || die "open [$file] for reading";
	while(<$fh>) {
		$DEBUG && print "CONF> $_";
		if (/^\s*$/) { next; }		# skip blank lines
		if (/^\s*#$/) { next; }		# skip comment lines
		if (/^\s*(\S+)\s*=\s*(\S+)/) {
			$varname = $1;
			$value = $2;
			$value =~ s/\${([^}]+)}/expand($1)/ge;
			$DEBUG && print "  setting [$varname] = [$value]\n";
			$VARS{$varname} = $value;
		}
	}
	$fh->close;
}

sub expand
{
	my $vname = shift;

	if (exists($VARS{$vname})) {
		return $VARS{$vname};
	} else {
		die "error in config-file\n";
	}
}

sub dumpVars
{
	foreach my $var (sort keys(%VARS)) {
		printf("%-18s %s\n", $var, $VARS{$var});
	}
}

1;

__END__

=head1 NAME

AbsenceConfig.pm - handles configuration for web-absence

=head1 SYNOPSIS

  use AbsenceConfig;

  AbsenceConfig::fetch('config-param');

=head1 DESCRIPTION

C(AbsenceConfig.pm> handles the configuration information that
belongs to a particular instance.  The module itself defines
'default' values for all configuration parameters.  If no
instance is explicitly configured, these 'default' values
are used.  If an instance is configured, the instance-specific
parameter has precedence, then the default value.

Currently understood parameters are:

=over 4

=item data_dir_rel

The relative directory path where data-files reside
(C<absence.db>, C<month_mod_times.db>, C<holiday.db>, C<absence-types.db>).
That is, the path as used in HTML source.

=item data_dir_abs

The absolute directory path where data-files reside.
That is, the path as used in CGI scripts.

=item cgi_dir_rel

The relative directory path where CGI-scripts and modules reside.

=item image_dir_rel

The relative directory path where the month-images and legend reside.

=item image_dir_abs

The absolute directory path where the month-images and legend reside.

=item database_file

The absolute path to the database file.

=item modtime_db_file

The absolute path to the modification time file.  This is used to hold
the modification times for month-images.  The modification time allows
URLs for the month-images to be constructed such that the browser
automatically reloads a month-image when it has been modified.

=item holiday_db_file

The absolute path to the holiday database file.

=item log_file

The absolute path to the log file.

=item type_file

The absolute path to the absence-type file.  This is where the various
kinds of absences can be defined along with a color.

=item main_script

The relative path to C<absence.pl>.

=item manage_script

The relative path to C<absence-manage.pl>.

=item control_script

The relative path to C<absence-control.pl>.

=item legend_style

The method used to arrive at the width for the legend boxes.  There are
two possibilities, C<fit> and C<contant:N> (where N is the width in pixels).
C<fit> uses the text to determine the width of the box. C<contant:N>
simply causes all boxes to have a width of N pixels.

=item backup_copies

Before the database is modified, a backup copy named C<absence.db.savN>
is made, where N is incremented each time.  This determines how many copies
are kept.

=item image_type

The format of the images used (legend and month-images).  The choices are
C<png>, C<jpg>, and C<gif>.  The version of GD that you installed must
support the format you choose here.

=item map_type

This determines what method to use for manipulating absences.  There
are two choices, C<server> (server-side image maps) and C<client>
(client-side image maps).  Client-side image maps allow the display
of so-called "tool-tips", but the the HTML generated is very large.
Server-side image maps result in much smaller HTML, but you must
click on absences and holidays to get more information about them.
Server-side image maps won't work on windoze/IIS.

=item cookie_domain

The domain information to be used when setting cookies.

=back

=head1 INSTANCES

absence now supports multiple "instances".  That is, completely
independent databases which all use the same Perl scripts and
modules.

=cut
