package AbsenceHolidays;

use FileHandle;

use AbsenceConfig;

BEGIN {
	$HOLIDAY_DB_FILE = AbsenceConfig::fetch('holiday_db_file');
	$INIT		= 0;
	#$DEBUG		= 1;
	$VERSION	= '1.2';
}

sub readHolidays
{
	my ($day, $month, $year, $desc);

	my $fh = FileHandle->new($HOLIDAY_DB_FILE);
	defined($fh) || die "open [$HOLIDAY_DB_FILE] for reading";
	while(<$fh>) {
		chomp;
		($DEBUG > 1) && print "HOLIDAY> $_\n";
		if (/^\s*$/) { next; }
		if (/^\s*#$/) { next; }
		if (/^(\d+)\.(\d+).(\d+|\*)\t(\S+(\s+\S+)*)\s*$/) {
			($day, $month, $year, $desc) = ($1, $2, $3, $4);
			$DEBUG && print "match: d=[$day], m=[$month], y=[$year], d=[$desc]\n";
			push(@{$DB{$month}->{$year}}, [$day, $month, $year, $desc]);
		}
	}
	$fh->close;

	$INIT = 1;
}

#-------------------------------------------------------------------
# getList()
#
# given a month and year, returns a *SORTED* list of holidays that
# fall within that month.  List is sorted ascending according to day
#-------------------------------------------------------------------
sub getList
{
	my ($month, $year) = @_;

	if (!$INIT) { readHolidays(); }

	my @out;

	if (!exists($DB{$month})) { return (); }

	if (exists($DB{$month}->{$year})) {
		foreach my $ref (@{$DB{$month}->{$year}}) {
			push(@out, $ref);
		}
	}
	if (exists($DB{$month}->{'*'})) {
		foreach my $ref (@{$DB{$month}->{'*'}}) {
			push(@out, $ref);
		}
	}

	return sort {$a->[0] <=> $b->[0]} @out;
}

1;

__END__

=head1 NAME

AbsenceHolidays.pm - primitive interface to a holiday database

=head1 SYNOPSIS

  use AbsenceHolidays;

  @holiday_list = AbsenceHolidays::getList($month, $year);

=head1 DESCRIPTION

B<AbsenceHolidays> provides a simple interface to a text file
containing dates and descriptions of holidays.  The format of the
file is:

  day.month.year description

for example:

  31.3.2002   Easter

if year equals '*' (asterisk), the holiday occurs on the same day each
year, otherwise a line for each year desired must appear in the file.
Blank lines and lines beginning with hash ('#') are ignored.

=head1 FUNCTIONS

=over 4

=item B<getList>

returns a list of all holidays present in a particular
month for a particular year.  The list returned consists of references
to a data-stucture:

  [ $day, $month, $year, $description ]

simple, eh?

=head1 AUTHOR

Robert Urban <urban@tru64.org>

Copyright (C) Robert Urban 2003

=cut
