package AbsenceImage;

use FileHandle;
use GD;
use Carp;

use AbsenceConfig;
use AbsenceDB;
use AbsenceDate;
use AbsenceHolidays;
use AbsenceTypes;

BEGIN {
	# some colors
	%COLOR_VALUES = (
		bg		=> [0xb0,	0xa0,	0xb0],
		we		=> [0x90,	0x80,	0x90],
		Holiday	=> [160,	32,		240],
		text	=> [   0,	   0,	   0],
		lines	=> [0xff,	0xff,	0xff],
		cw		=> [0xaa,	0x80,	0xaa],
		padding	=> [0x60,	0x50,	0x60],
	);
	%COLOR = ();
	$ROW_HEIGHT			= 20;
	$COL_WIDTH			= 20;
	$ROW_LABEL_WIDTH	= 100;
	$HEADER_LINES		= 2;
	$CGI_DIR			= AbsenceConfig::fetch('cgi_dir_rel');

	$DEBUG		= 0;
	$VERSION	= '1.3';
}

sub dimensions
{
	my ($month, $year, $group_id) = @_;

	my ($width, $height);

	# calculate width of image
	$width = $ROW_LABEL_WIDTH + 31 * $COL_WIDTH + 1;

	# get people
	my @people = AbsenceDB::getPeople($group_id);

	# calculate height of image
	$height = (@people + 2) * $ROW_HEIGHT + 1;

	($width, $height);
}

sub create
{

	my ($month, $year, $group_id, $outfile) = @_;

	if (!$month || !$year) {
		confess "missing people, month, or year";
	}

	my $numdays = AbsenceDate::daysInMonth($month, $year);

	# get people
	my @people = AbsenceDB::getPeople($group_id);

	# calculate width of image
	my $use_width = $ROW_LABEL_WIDTH + $numdays * $COL_WIDTH;
	my $i_width = $ROW_LABEL_WIDTH + 31 * $COL_WIDTH + 1;

	# calculate height of image
	$I_HEIGHT = (@people + 2) * $ROW_HEIGHT + 1;

	my $im = new GD::Image($i_width, $I_HEIGHT);

	allocateColors($im);
	#$im->transparent($COLOR{bg});

	# do background
	$im->filledRectangle(0, 0, $i_width, $I_HEIGHT, $COLOR{bg});

	# if numdays != 31, add padding color
	if ($numdays != 31) {
		my $x1 = $ROW_LABEL_WIDTH + $numdays * $COL_WIDTH + 1;
		#print "<BR>X1 = [$x1], X2 = [$i_width]<BR>\n";
		$im->filledRectangle(
			$x1, 0,
			$i_width, $I_HEIGHT,
			$COLOR{padding});
	}

	# mark weekends
	markWeekends($im, $month, $year);
	
	# mark holidays
	markHolidays($im, $month, $year);

	# add month label
	$im->string(gdGiantFont, 10, 5, AbsenceDate::monthName($month), $COLOR{text});
	$im->string(gdGiantFont, 10, 20, $year, $COLOR{text});

	# add CW bars
	drawCwBars($im, $month, $year);

	# draw grid
	my $cur_y = $sav = $HEADER_LINES * $ROW_HEIGHT;
	my $end = scalar(@people) + 1;
	#print "end = $end\n";
	for(my $i = 0; $i <= $end; $i++) {
		#print "line: 0, $cur_y, $i_width, $cur_y\n";
		$im->line(0, $cur_y, $use_width, $cur_y, $COLOR{lines});
		$cur_y += $ROW_HEIGHT;
	}

	$cur_x = $ROW_LABEL_WIDTH;
	for($i = 0; $i <= $numdays; $i++) {
		#print "line: $cur_x, $sav, $cur_x, $I_HEIGHT\n";
		$im->line($cur_x, $sav, $cur_x, $I_HEIGHT, $COLOR{lines});
		$cur_x += $COL_WIDTH;
	}

	# add monthdays
	$cur_x = $ROW_LABEL_WIDTH;
	for($i = 1; $i <= $numdays; $i++) {
		$im->string(gdSmallFont, $cur_x + 3, $ROW_HEIGHT, $i, $COLOR{text});
		$cur_x += $COL_WIDTH;
	}

	# add people
	$cur_y = $sav;
	foreach my $person_id (@people) {
		my $name = AbsenceDB::getPerson($person_id);
		if (length($name) > 15) {
			$name = substr($name, 0, 15);
		}
		$im->string(gdSmallFont, 5, $cur_y + 3, $name, $COLOR{text});
		$cur_y += $ROW_HEIGHT;
	}

	# add absences
	my $rid;
	$DEBUG && print "adding absences..\n";
	$cur_y = $sav;
	foreach my $pid (@people) {
		$DEBUG && print "getting month absences for person_id [$pid]\n";
		my @res_list = AbsenceDB::getMonthReservations($pid, $month, $year);
		foreach $res (@res_list) {
			my ($rid, $start, $end, $type, $desc) = @$res;
			$DEBUG && print "got res=$res, start=$start, end=$end\n";
			addReservation($im, $cur_y, $start, $end, $type, $desc, $rid);
		}
		$cur_y += $ROW_HEIGHT;
	}

	open(OUT, ">$outfile") || die "open";
	binmode(OUT);	# stoopid windoze
	my $it = AbsenceConfig::fetch('image_type');
	if ($it eq 'png') {
		print OUT $im->png;
	} elsif ($it eq 'gif') {
		print OUT $im->gif;
	} elsif ($it eq 'jpg') {
		print OUT $im->jpg;
	} else {
		die "unknown image_type [$it]";
	}
	close(OUT);
}

#---------------------------------------------------------------
# findAbsence()
#
# is passed month, year, and (x,y) coordinates of click
#
# returns array of two elements.  The first can be 'none', 'absence',
# 'holiday' or 'start'.  The second depends on the first.
#
# The values mean:
# 'none'		user clicked on the first line (CW, or on the second
#				line (month-day) but not on a holiday
# 'absence'		user clicked on a marked absence
# 'holiday'		user clicked on a month-day marked as holiday
# 'start'		user clicked on a user-line not marked as absence
#
# The second element is:
# 'none'		blank
# 'absence'		the absence-ID
# 'holiday'		returns ref to a holiday array in format:
#				[<day>, <month>, <year>, <holiday-name>]
# 'start'		a reference to an array containing the person_id and date
#				corresponding to the click, in format
#				[<person_id>,<D>,<M>,<Y>]
#---------------------------------------------------------------
sub findAbsence
{
	my ($month, $year, $gid, $x, $y) = @_;

	my $numdays = AbsenceDate::daysInMonth($month, $year);
	my $sav = $HEADER_LINES * $ROW_HEIGHT;

	# sanity check
	if ($x < $ROW_LABEL_WIDTH) {
		return (none => '');
	}

	# check if click in padding
	if ($x > ($ROW_LABEL_WIDTH + $numdays * $COL_WIDTH)) {
		return (none => 'stop-that');
	}

	# get row number from y-coord
	my $row = int($y/$ROW_HEIGHT);		# returns row 0-n

	if ($row == 1) {
		# month-day row
		my $hol = findHoliday($month, $year, $x);
		if (defined($hol)) {
			return (holiday => $hol);
		}
		return (none => 'no-holiday');
	}

	# at this point, coordinates must be in a person row
	if ($y <= $sav) {
		return (none => '');
	}

	# get people
	my @people = AbsenceDB::getPeople($gid);
	my $person_id = $people[$row - $HEADER_LINES];

	my ($start, $end, $short_desc, $x1, $x2);

	# go through absences
	my $rid;
	$DEBUG && print "finding absence: $month/$year, ($x,$y)..\n";
	my @res_list = AbsenceDB::getMonthReservations($person_id, $month, $year);
	foreach $res (@res_list) {
		($rid, $start, $end, $short_desc) = @$res;
		$x1 = $ROW_LABEL_WIDTH + ($start - 1) * $COL_WIDTH + 1;
		$x2 = $ROW_LABEL_WIDTH + $end * $COL_WIDTH - 1;
		$DEBUG && print "checking id=$rid, x1=$x1, x2=$x2...\n";
		if (($x >= $x1) && ($x < $x2)) {
			return (absence => $rid);
		}
	}
	#$cur_y += $ROW_HEIGHT;

	# must be a click on a person-row but not on absence...
	# figure out date

	$x -= $ROW_LABEL_WIDTH;
	my $day = int($x/$COL_WIDTH) + 1;
	return (start => [$person_id, $day, $month, $year]);
}

sub markWeekends
{
	my ($im, $month, $year) = @_;

	my $numdays = AbsenceDate::daysInMonth($month, $year);

	my ($x1, $y1, $x2, $y2, $start, $end);

	#$y1 = $HEADER_LINES * $ROW_HEIGHT;
	$y1 = 0;
	$y2 = $I_HEIGHT;


	# find first sat/sun
	my $i = 1;
	my $wd;
	while(($wd = AbsenceDate::weekDayFromDate($i, $month, $year)) % 6 != 0) {
		$i++;
	}
	$start = $i;
	$end = ($wd == 6) ? $i + 1 : $i;

	do {
		$x1 = $ROW_LABEL_WIDTH + ($start - 1) * $COL_WIDTH + 1;
		$x2 = $ROW_LABEL_WIDTH + $end * $COL_WIDTH - 1;
		$im->filledRectangle($x1, $y1, $x2, $y2, $COLOR{we});
		$start = $end + 6;
		$end = (($start + 1) <= $numdays) ? $start + 1 : $start;
	} while($start <= $numdays);
}

sub findHoliday
{
	my ($month, $year, $x) = @_;

	my $numdays = AbsenceDate::daysInMonth($month, $year);
	my @holidays = AbsenceHolidays::getList($month, $year);

	if (!@holidays) { return undef; }

	my ($x1, $x2);

	my $i;
	for($i = 1; $i <= $numdays; $i++) {
		if ($i == $holidays[0]->[0]) {
			$x1 = $ROW_LABEL_WIDTH + ($i - 1) * $COL_WIDTH + 1;
			$x2 = $ROW_LABEL_WIDTH + $i * $COL_WIDTH - 1;
			if (($x >= $x1) && ($x <= $x2)) {
				$holidays[0]->[2] = $year;
				return $holidays[0];
			}
			shift(@holidays);
			if (!@holidays) { last; }
		}
	}

	return undef;
}

sub markHolidays
{
	my ($im, $month, $year) = @_;

	my $numdays = AbsenceDate::daysInMonth($month, $year);
	my @holidays = AbsenceHolidays::getList($month, $year);

	if (!@holidays) { return; }

	my ($x1, $y1, $x2, $y2, $start, $end);

	$y1 = $ROW_HEIGHT;
	$y2 = $HEADER_LINES * $ROW_HEIGHT;

	my $i;
	for($i = 1; $i <= $numdays; $i++) {
		if ($i == $holidays[0]->[0]) {
			$x1 = $ROW_LABEL_WIDTH + ($i - 1) * $COL_WIDTH + 1;
			$x2 = $ROW_LABEL_WIDTH + $i * $COL_WIDTH - 1;
			$im->filledRectangle($x1, $y1, $x2, $y2, $COLOR{Holiday});
			shift(@holidays);
			if (!@holidays) { last; }
		}
	}
}

sub allocateColors
{
	my $im = shift;

	# retrieve absence types and colors and then allocate colors
	my $types = AbsenceTypes::getHash();

	# allocate colors
	
	foreach my $col (keys(%COLOR_VALUES)) {
		$COLOR{$col} = $im->colorAllocate(@{$COLOR_VALUES{$col}});
	}

	foreach my $type (keys(%$types)) {
		$COLOR{$type} = $im->colorAllocate(@{$types->{$type}});
	}
}

sub drawCwBars
{
	my ($image, $month, $year) = @_;

	my $monthdays = AbsenceDate::daysInMonth($month, $year);
	my ($cw, $cy, $x1, $y1, $x2, $y2);

	my $wd;
	my $day = 1;
	while(
		(($wd = AbsenceDate::weekDayFromDate($day, $month, $year)) == 0) ||
		($wd == 6)
	)
	{
		$day++;
	}
	$DEBUG && print "day = $day\n";

	# $day is first monday - friday
	while($day <= $monthdays) {
		$x1 = $ROW_LABEL_WIDTH + ($day - 1) * $COL_WIDTH + 1;
		$y1 = 1;
		my $end;
		if (($day > 1) || ($wd == 1)) {
			$end = (($day + 4) <= $monthdays)
				? 4
				: $monthdays - $day;
		} else {
			$end = 5 - $wd;
		}
		$x2 = $ROW_LABEL_WIDTH + ($day + $end) * $COL_WIDTH - 1;
		$y2 = $ROW_HEIGHT - 1;
		$image->filledRectangle($x1, $y1, $x2, $y2, $COLOR{cw});
		if (($wd == 5) && ($day == 1)) {
			# if first day is friday, no room for CW-label
			$day += 3;
			$wd = 0;
			next;
		}
		($cw, $cy) = AbsenceDate::calendarWeek($day, $month, $year);
		my $mid = int(($x2 - $x1)/2);
		$image->string(gdSmallFont,
			$x1 + $mid - 9,
			3,
			"CW $cw",
			$COLOR{text});
		if ($wd) {
			$day += 8 - $wd;
			$wd = 0;
		} else {
			$day += 7;
		}
	}
}

#------------------------------------------------------------------
# addReservation()
#
# $start and $end are the month-day numbers within the month in
# question
#------------------------------------------------------------------
sub addReservation
{
	my ($im, $y_offset, $start, $end, $type, $desc, $rid) = @_;

	my ($x1, $y1, $x2, $y2);
	$x1 = $ROW_LABEL_WIDTH + ($start - 1) * $COL_WIDTH + 1;
	$y1 = $y_offset + 1;
	$x2 = $ROW_LABEL_WIDTH + $end * $COL_WIDTH - 1;
	$y2 = $y_offset + $ROW_HEIGHT - 1;

	$DEBUG && print "  ABS: ($x1, $y1), ($x2, $y2) - $type, ID=$rid\n";

	$im->filledRectangle($x1, $y1, $x2, $y2, $COLOR{$type});
	my $mid = int(($x2 - $x1)/2);
	
	#-----------------------------------------------------------------
	# experience show that approximately 3 characters in the chosen
	# label font fit in a column of width 20 ($COL_WIDTH). Yeah, it's
	# a hack.
	#-----------------------------------------------------------------

	my $num_days = $end - $start + 1;

	my ($half, $lab_xpos);

	if (int((length($desc)+2)/3) >= $num_days) {
		$lab_xpos = $x1 + 2;
		$desc = substr($desc, 0, $num_days*3);
	} else {
		$half = int(length($desc)/6.0 * $COL_WIDTH);
		$lab_xpos = $x1 + $mid - $half;
	}



	#if (int(length($type)/3) >= $num_days) {
	#	$lab_xpos = $x1 + 1;
	#	$type = substr($type, 0, $num_days*3);
	#}

	$im->string(gdSmallFont,
		$lab_xpos,
		$y_offset + 3,
		$desc,
		$COLOR{text});
}

sub makeLegend
{
	my $outfile = shift;

	my $type;
	my $tag_width;

	my $style = AbsenceConfig::fetch('legend_style');
	if ($style =~ /^constant:(\d+)$/) {
		$tag_width = $1;
		$style = 'constant';
	} elsif ($style ne 'fit') {
		die "unrecognized legend-style [$style]";
	}

	# check if legend needs to be produced.
	if (-e $outfile) {
		my $type_mtime = AbsenceTypes::modTime();
		my $img_mtime = (stat($outfile))[9];
		if ($type_mtime > $img_mtime) {
			# older
			#print "OLDER.<BR>\n";
			return;
		}
	}

	my $types = AbsenceTypes::getHash();
	my $num_types = scalar(keys(%$types)) + 1;

	my $tag_sep = 20;
	my $i_width = 60;
	my @list;
	foreach $type (keys(%$types), 'Holiday') {
		if ($style eq 'constant') {
			push(@list, [$type, $tag_width]);
			$i_width += $tag_width + $tag_sep;
		} else {
			my $w = length($type) * 6 + 5;
			push(@list, [$type, $w]);
			$i_width += $w + $tag_sep;
		}
	}

	my $i_height = 25;

	my $im = new GD::Image($i_width, $i_height);

	allocateColors($im);

	# do background
	$im->filledRectangle(0, 0, $i_width, $i_height, $COLOR{bg});

	$im->string(gdSmallFont,
		2, 5,
		'Legend:',
		$COLOR{text});

	my $x = 60;

	my $width;
	foreach my $ref (@list) {
		($type, $width) = @$ref;
		$im->filledRectangle($x, 4, $x+$width, $i_height - 4, $COLOR{$type});
		$im->string(gdSmallFont,
			$x+2, 6,
			$type,
			$COLOR{text});
		$x += $width + $tag_sep;
	}

	my $fh = FileHandle->new(">$outfile");
	defined($fh) || die "open $outfile for writing";
	binmode($fh);	# stoopid windoze
	my $it = AbsenceConfig::fetch('image_type');
	if ($it eq 'png') {
		print $fh $im->png;
	} elsif ($it eq 'gif') {
		print $fh $im->gif;
	} elsif ($it eq 'jpg') {
		print $fh $im->jpg;
	} else {
		die "unknown image_type [$it]";
	}
	close($fh);
}

sub genClientMap
{
	my ($gid, $month, $year) = @_;

	my $numdays = AbsenceDate::daysInMonth($month, $year);

	# get people
	my @people = AbsenceDB::getPeople($gid);

	# calculate width of image
	my $use_width = $ROW_LABEL_WIDTH + $numdays * $COL_WIDTH;
	my $i_width = $ROW_LABEL_WIDTH + 31 * $COL_WIDTH + 1;

	# calculate height of image
	$I_HEIGHT = (@people + 2) * $ROW_HEIGHT + 1;

	my $inst;
	if (exists($ENV{INSTANCE_NAME})) {
		$inst = "/in=$ENV{INSTANCE_NAME}";
	}

	#-----------------------------------------------------------
	# prologue
	#-----------------------------------------------------------
	my $name = "g${gid}m${month}y${year}";
	print qq[<MAP NAME="$name">\n];

	#-----------------------------------------------------------
	# create holiday areas
	#-----------------------------------------------------------
	my @holidays = AbsenceHolidays::getList($month, $year);

	my ($x1, $y1, $x2, $y2, $start, $end);

	if (@holidays) {
		$y1 = $ROW_HEIGHT;
		$y2 = $HEADER_LINES * $ROW_HEIGHT;

		my $i;
		for($i = 1; $i <= $numdays; $i++) {
			if ($i == $holidays[0]->[0]) {
				$x1 = $ROW_LABEL_WIDTH + ($i - 1) * $COL_WIDTH + 1;
				$x2 = $ROW_LABEL_WIDTH + $i * $COL_WIDTH - 1;
				print qq[<AREA SHAPE="rect" COORDS="$x1,$y1,$x2,$y2" NOHREF TITLE="$holidays[0]->[3]">\n];
				shift(@holidays);
				if (!@holidays) { last; }
			}
		}
	}

	#-----------------------------------------------------------
	# create areas for either non-reserved days or for absences
	#-----------------------------------------------------------
	my $rid;
	my $cur_y = $HEADER_LINES * $ROW_HEIGHT;
	my $title;
	my $target;

	$DEBUG && print "creating people areas..\n";
	foreach my $pid (@people) {

		$y1 = $cur_y + 1;
		$y2 = $cur_y + $ROW_HEIGHT - 1;

		my $p = AbsenceDB::getPerson($pid);
		
		print "<!-- checking absences for $p, person_id [$pid] -->\n";

		$DEBUG && print "getting month absences for $p, person_id [$pid]\n";
		my @res_list = AbsenceDB::getMonthReservations($pid, $month, $year);
		my @sorted = sort {$a->[1] <=> $b->[1]} @res_list;
		my $day = 1;
		while($day <= $numdays) {
			$x1 = $ROW_LABEL_WIDTH + ($day - 1) * $COL_WIDTH + 1;
			if (@sorted && ($sorted[0]->[1] == $day)) {
				my ($rid, $start, $end, $type, $desc) = @{$sorted[0]};
				$x2 = $ROW_LABEL_WIDTH + $end * $COL_WIDTH - 1;
				$title = qq{TITLE="$desc"};
				$target = "res,$rid";
				shift(@sorted);
				$day = $end + 1;
			} else {
				$x2 = $ROW_LABEL_WIDTH + $day * $COL_WIDTH - 1;
				$title = '';
				$target = "new,$pid,$gid,$day,$month,$year";
				$day++;
			}
			$target = qq[HREF="$CGI_DIR/absence-click.pl$inst?$target"];
			print qq[<AREA SHAPE="rect" COORDS="$x1,$y1,$x2,$y2" $target $title>\n];
		}
		$cur_y += $ROW_HEIGHT;
	}

	#-----------------------------------------------------------
	# epilogue
	#-----------------------------------------------------------
	print "</MAP>\n";

	return $name;
}

1;

__END__

=head1 NAME

AbsenceImage.pm - handles manipulation of month-images and related functions

=head1 SYNOPSIS

  ($width, $height) = AbsenceImage::dimensions($month, $year, $group_id);
  AbsenceImage::create($month, $year, $group_id, $output_file_name);
  $arr_ref = AbsenceImage::findAbsence($month, $year, $group_id, $x, $y);
  $holiday_struct = AbsenceImage::findHoliday($month, $year, $x);
  $map_name = AbsenceImage::genClientMap($group_id, $month, $year);

=head1 DESCRIPTION

C<AbsenceImage.pm> presents an interface to month-image data.  It
handles the generation of month-images and also the querying of
absence-data based on an (X,Y) coordinate-pair (see C<findAbsence()>,
C<findHoliday()>).

=head1 FUNCTIONS

=over 4

=item B<dimensions()>

takes $month, $year, and $group_id as parameters. Returns a list
consisting of the width and height of the corresponding month-image.

=item B<create()>

generates a month-image, which is written to output file specified.

=item B<findAbsence()>

This is used when server-side image maps have been configured.
given a (X,Y) coordinate pair, determines what, if any, absence
entity corresponds to the pair, and returns a corresponding
data-structure as a list.  The possible return values are:

=over 4

=item (none => '')

no entity exists matching the supplied coordinates

=item (none => 'stop-that')

coordinates correspond to padding at end of month with less than 31 days.

=item (none => 'no-holiday')

coordinates correspond to row holding day-numbers, but not to a holiday.

=item (holiday => [$day, $month, $year, $holiday_description])

coordinates correspond to row holding day-numbers and to a holiday.

=item (absence => $absence_id)

coordinates correspond to an existing absence.

=item (start => [$person_id, $day, $month, $year])

coordinates correspond to a valid unreserved person/day combination.

=back

=item B<findHoliday()>

findHoliday() isn't really public.  It's normally used by findAbsence().
Takes a $month, $year, and $x coordinate, and returns an array reference
if the $x coordinate corresponds to a holiday, or B<undef> if not.

=item B<genClientMap()>

B<genClientMap()> is responsible for generating a single client-side
image map corresponding to a particular month.
B<genClientMap()> is used when client-side image maps have been
configured.  It takes ($group_id, $month, $year) as parameters.
The image map is written directly to STDOUT.

=back

=head1 AUTHOR

Robert Urban <urban@tru64.org>

Copyright (C) 2003 Robert Urban

=cut
