package AbsenceLog;

use Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(abslog);

use Fcntl (':flock'); # import LOCK_* constants
use FileHandle;

use AbsenceConfig;

BEGIN {
	$LOGFILE	= AbsenceConfig::fetch('log_file') ||
		die "no logfile configured.\n";
	$IPADDR		= $ENV{'REMOTE_ADDR'};
}

sub SEEK_END() { 2; }	# not portable, but what the hell

sub abslog
{
	my $msg = shift;

	my $ts = timestamp();

	my $openstr = "+<$LOGFILE";

	if (! -e $LOGFILE) {
		$openstr = ">$LOGFILE";
	}

	my $fh = FileHandle->new($openstr);
	defined($fh) || die "open of [$LOGFILE] failed";
	flock($fh, LOCK_EX) || die "flock lock_ex on $LOGFILE ($!)";
	# position file-pointer at EOF
	seek($fh, 0, SEEK_END);
	if (ref($msg)) {
		print $fh "[$ts / $IPADDR]\n";
		foreach (@$msg) {
			print $fh "\t$_\n";
		}
	} else {
		print $fh "[$ts / $IPADDR] $msg\n";
	}
	flock($fh, LOCK_UN) || die "flock lock_un on $LOGFILE ($!)";
	$fh->close;
}

sub timestamp
{
	my @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	my ($sec,$min,$hour,$mday,$mon,$year,@junk) = localtime(time);

	return sprintf("%d-%s-%d %02d:%02d:%02d",
		$mday, $months[$mon], $year+1900, $hour, $min, $sec);
}

1;

__END__

=head1 NAME

AbsenceLog.pm - primitive logging facility for absence

=head1 SYNOPSIS

  use AbsenceLog;

  abslog('text to be logged, not CR-terminated');

=head1 DESCRIPTION

AbsenceLog.pm provides a simple logging facility.  Messages are
written to the logfile specified in by 'log_file' in the configuration.
There is no provision for not having a logfile, i.e., a C<use AbsenceLog>
will fail if 'log_file' is not configured.  The log file must also exist.

=head1 FUNCTIONS

=over 4

=item abslog()

Takes a single parameter consisting of text to be logged.  The text
should not be carriage-return terminated (that is done automatically).
B<abslog> is exported by default for convenience.

=back

=head1 AUTHOR

Robert Urban <urban@tru64.org>

Copyright (C) Robert Urban 2003

=cut
