#!/usr/bin/perl

#==========================================================================
# Welcome to "absence-click.pl"!
#
# It's a bit spaghetti-ish. Sorry about that.
# This script is a bit complicated because it is called at two
# stages.  First, it is called when a user clicks upon a month-image.
# In this case, the tests for the existence of the show_absence and
# create_absence CGI parameters fail, and processing starts at the
# point marked "First entry-point".
# The click-information is analyzed and then one of
#   badClick()
#   showAbsence()
#   showHoliday()
#   newAbsence()
# is called (depending on whether client-side or server-side image-maps
# have been configured).
#
# These functions display a form whose target is this script.
# When the users clicks on the SUBMIT button in one of the forms,
# the appropriate CGI parameters are set and this script is re-entered.
#
# The script performs the requested operation (adding a new reservation,
# for example), and either causes a browswer redirect to the main
# page (absence.pl) on success, or causes an error message to be
# displayed.
#
# Everything is slightly complicated by the necessity of have the
# "INSTANCE_NAME" environmental-variable set (if applicable) before
# the AbsenceConfig module is loaded. The instance-information must
# be passed to this script, and the method used to pass it is
# different for stages 1 and 2.  In stage 1, the instance-name
# is passed as part of the "extended path information" contained
# in the URL.  In stage 2, the instance-name is passed as a
# CGI parameter.
#==========================================================================

use CGI;

BEGIN {
	$q = new CGI;
	$q->import_names;

	#system("echo 'PATH_INFO=$ENV{PATH_INFO}' > /tmp/foo.xx");
	if (exists($ENV{PATH_INFO}) && ($ENV{PATH_INFO} =~ m!/in=([^/?]+)!)) {
		$ENV{INSTANCE_NAME} = $1;
		#system("echo 'instance=$1' >> /tmp/foo.xx");
	} elsif (defined($Q::instance)) {
		$ENV{INSTANCE_NAME} = $Q::instance;
	}
}

use AbsenceConfig;
use AbsenceImage;
use AbsenceDB;
use AbsenceTypes;

%MONTH_LABELS = (
	1	=> 'January',
	2	=> 'February',
	3	=> 'March',
	4	=> 'April',
	5	=> 'May',
	6	=> 'June',
	7	=> 'July',
	8	=> 'August',
	9	=> 'September',
	10	=> 'October',
	11	=> 'November',
	12	=> 'December',
);
$DEBUG = 0;

# various paths to scripts and pages
$MAIN_SCRIPT = AbsenceConfig::fetch('main_script');
$VERSION	= '1.3';

$CGI::POST_MAX = 300;
$CGI::DISABLE_UPLOADS = 1;

if (exists($ENV{INSTANCE_NAME})) {
	$MAIN_SCRIPT .= "?instance=$ENV{INSTANCE_NAME}";
}

if (defined($Q::show_absence)) {
	if ($Q::show_absence =~ /^save/i) {
		modifyAbsence($q);
		exit;
	} elsif ($Q::show_absence eq 'Delete') {
		deleteAbsence($q);
		exit;
	} elsif ($Q::show_absence eq 'Cancel') {
		print $q->redirect($MAIN_SCRIPT);
		exit(0);
	} else {
		print "whoops!\n";
	}
}

if (defined($Q::create_absence)) {
	createAbsence($q);
	exit(0);
}

#-------------------------------------------------------------------
# First entry-point.
#
# if I got here, it means the script has been called after a user
# clicked on one of the month-images
#-------------------------------------------------------------------
print $q->header();

@keywords = $q->keywords;
$stuff = $keywords[0];

#-------------------------------------------------------------------
# The encoding of absence information depends on whether map_type
# is set to 'client' or 'server'
#-------------------------------------------------------------------
if (AbsenceConfig::fetch('map_type') eq 'server') {
	if ($ENV{'PATH_INFO'} !~ m!^(/in=[^/]+)/(\d+)-(\d+)-(\d+)$!) {
		die "failed to match [$ENV{'PATH_INFO'}]";
	}
	$gid = $2;
	$month = $3;
	$year = $4;
	if ($stuff =~ m!^(\d+),(\d+)$!) {
		$x = $1;
		$y = $2;
		#print "month [$month], year [$year], x [$x], y [$y]<BR>\n";
	} else {
		print "map_type = server, unable to parse [$stuff]<BR>\n";
	}
	@result = AbsenceImage::findAbsence($month, $year, $gid, $x, $y);
	$key = $result[0];
} else {
	if ($stuff =~ /res,(\d+)$/) {
		@result = (absence => $1);
		$key = 'absence';
	} elsif ($stuff =~ /new,(\d+),(\d+),(\d+),(\d+),(\d+)$/) {
		($pid,$gid,$day,$month,$year) = ($1, $2, $3, $4, $5);
		@result = (start => [$pid, $day, $month, $year]);
		$key = 'start';
	} else {
		print "map_type = client, unable to parse [$stuff]\n";
	}
}

if ($key eq 'none') {
	badClick($q, $result[1]);
} elsif ($key eq 'absence') {
	showAbsence($q, \@result);
} elsif ($key eq 'holiday') {
	showHoliday($q, \@result);
} elsif ($key eq 'start') {
	newAbsence($q, \@result);
} else {
	print "findAbsence error. returned: [@result]<BR>\n";
}

#print "<P>ENV:<BR>\n";
#foreach my $key (keys(%ENV)) {
#	print "[$key] $ENV{$key}<BR>\n";
#}

if ($DEBUG) {
	@names = $q->param;

	print "Parameters:<BR>\n";
	foreach $p (@names) {
		print " [$p]<BR>\n";
	}

	print "<P>Keywords:<BR>\n";
	foreach $p (@keywords) {
		print " [$p]<BR>\n";
	}
}

print $q->end_html();

#====================================================================
# subs
#====================================================================

#------------------------------------------------------------------
# deleteAbsence()
#
# user has clicked on "Delete" in the show/modify/delete form
#------------------------------------------------------------------

sub deleteAbsence
{
	my $q = shift;
	print "<H1>Delete an Absence:</H1><BR>\n";

	my $msg;

	my $ret = AbsenceDB::deleteReservation($Q::res_id);

	if ($ret eq 'ok') {
		print $q->redirect($MAIN_SCRIPT);
		#print "Successfully completed.<BR>\n";
	} elsif ($ret eq 'conflict') {
		$msg = "Looks like someone else got there first.<BR>\n";
	} elsif ($ret eq 'internalerror') {
		$msg = "internal error. (sorry)<BR>\n";
	}

	print $q->header();
	print $q->start_html(
		-title => 'Absence: delete absence',
		-target => 'display',
	);
	print $msg;
	print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
	print $q->submit(action => 'Jump back to Calendar');
	print $q->end_form();
	print $q->end_html();
}

#------------------------------------------------------------------
# modifyAbsence()
#
# user has clicked on "Save Changes" in the show/modify/delete form
#------------------------------------------------------------------

sub modifyAbsence
{
	my $q = shift;
	my $start = "$Q::sday.$Q::smonth.$Q::syear";
	my $end = "$Q::eday.$Q::emonth.$Q::eyear";

	my $msg;

	my $ret = AbsenceDB::addReservation($Q::res_id, $Q::person_id,
		$start, $end, $Q::type, $Q::desc);

	#print "AbsenceDB::addReservation returned [$ret]<BR>\n";

	if ($ret eq 'ok') {
		print $q->redirect($MAIN_SCRIPT);
		#print "<P>Absence successfully modified.<BR>\n";
	} elsif ($ret eq 'impossible') {
		$msg = "<P>ERROR: The requested absence ends before it starts!\n";
	} elsif ($ret eq 'conflict') {
		$msg = "<P>ERROR: The requested absence would overlap with an existing one\n".
		"<BR>Please select a new (ending?) date.<BR>\n";
	}

	print $q->header();
	print $q->start_html(
		-title => 'Absence: modify an existing absence',
		-target => 'display',
	);

	print "<H1>Modify an Absence:</H1><BR>\n";

	print $msg;

	print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
	print $q->submit(action => 'Jump back to Calendar');
	print $q->end_form();
	print $q->end_html();
}

#------------------------------------------------------------------
# newAbsence()
#
# display a form to allow the create of a new absence
#------------------------------------------------------------------

sub createAbsence
{
	my $q = shift;

	if ($Q::create_absence eq 'Cancel') {
		print $q->redirect($MAIN_SCRIPT);
		exit(0);
	}

	my $start = "$Q::sday.$Q::smonth.$Q::syear";
	my $end = "$Q::eday.$Q::emonth.$Q::eyear";

	my $msg;

	my $ret = AbsenceDB::addReservation('new', $Q::person_id, $start, $end,
		$Q::type, $Q::desc);

	print "AbsenceDB::addReservation returned [$ret]<BR>\n";
	if ($ret eq 'ok') {
		print $q->redirect($MAIN_SCRIPT);
		#print "<P>Absence successfully created.<BR>\n";
	} elsif ($ret eq 'impossible') {
		$msg = "<P>ERROR: The requested absence ends before it starts!\n";
	} elsif ($ret eq 'conflict') {
		$msg = "<P>ERROR: The requested absence would overlap with an existing one\n".
		"<BR>Please select a new (ending?) date.<BR>\n";
	} elsif ($ret eq 'badday') {
		$msg = "<P>ERROR: start or end date invalid (past end of month)\n";
	}

	print $q->header();
	print $q->start_html(
		-title => 'Absence: create a new absence',
		-target => 'display',
	);
	print "<H1>Create Absence Error</H1><BR>\n";

	print $msg;

	print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
	print $q->submit(action => 'Jump back to Calendar');
	print $q->end_form();
	print $q->end_html();
}

sub newAbsence
{
	my ($q, $rref) = @_;

	my $sref = $rref->[1];

	print $q->start_html(-title => 'Absence: create a new absence',
		-bgcolor => 'teal');

	print $q->h1("Create a new  absence");

	absenceForm(new => $sref);

	print $q->submit(create_absence => 'Save Changes'),
		$q->submit(create_absence => 'Cancel');

	print $q->end_form;
}

sub showAbsence
{
	my ($q, $rref) = @_;

	print $q->start_html('Absence: display/modify an absence');

	print $q->h1("Display or Modify an absence");

	my $res_id = $rref->[1];
	my $res = AbsenceDB::getReservation($res_id);
	if (!defined($res)) {
		print "error: reservation id [$res_id] not found.<BR>\n";
	} else {
		absenceForm(show => $res);
		print $q->submit(show_absence => 'Save Changes'),
			$q->submit(show_absence => 'Cancel'),
			$q->submit(show_absence => 'Delete');
		print $q->end_form;
	}
}

sub absenceForm
{
	my ($type, $ref) = @_;

	my ($pid);

	# handle default values
	my %def;

	if ($type eq 'show') {
		$pid = $ref->{person_id};
		$def{sday}		= $ref->{start}->[0];
		$def{smonth}	= $ref->{start}->[1];
		$def{syear}		= $ref->{start}->[2];

		$def{eday}		= $ref->{end}->[0];
		$def{emonth}	= $ref->{end}->[1];
		$def{eyear}		= $ref->{end}->[2];

		$def{type}		= $ref->{type};
		$def{desc}		= $ref->{desc};
		$def{res_id}	= $ref->{id};
	} else {
		#my $start	= $ref->{start};
		#my $end		= $ref->{end};

		$pid = $ref->[0];
		$def{sday}		= $ref->[1];
		$def{smonth}	= $ref->[2];
		$def{syear}		= $ref->[3];
		$def{eday}		= $ref->[1];
		$def{emonth}	= $ref->[2];
		$def{eyear}		= $ref->[3];
		$def{type}		= AbsenceTypes::getDefault();
		$def{desc}		= '';
		$def{res_id}	= '';
	}

	my $bg = 'papayawhip';

	my $person = AbsenceDB::getPerson($pid);

	#my $start = "$sref->[1]/$sref->[2]/$sref->[3]";
	#print "Add an absence for [$person] starting on $start.<BR>\n";

	print $q->start_form;
	print $q->hidden(-name => 'person_id', -default => $pid);
	print $q->hidden(-name => 'res_id', -default => $def{res_id});
	if (exists($ENV{INSTANCE_NAME})) {
		print $q->hidden(-name => 'instance', -default => $ENV{INSTANCE_NAME});
	}
	print "<TABLE>\n";
	#------------------------------------------------
	# person row
	#------------------------------------------------
	print "<TR><TH ALIGN=right BGCOLOR=$bg>Person</TH><TD>$person</TD></TR>\n";
	#------------------------------------------------
	# begin row
	#------------------------------------------------
	print "<TR><TH ALIGN=right BGCOLOR=$bg>Begin</TH><TD>";
	print $q->popup_menu(
		-name	=> 'sday',
		-values	=> [1..31],
		-default=> $def{sday},
	);
	print $q->popup_menu(
		-name	=> 'smonth',
		-values	=> [1..12],
		-default=> $def{smonth},
		-labels	=> \%MONTH_LABELS,
	);
	print $q->popup_menu(
		-name	=> 'syear',
		-values	=> [2002..2010],
		-default=> $def{syear},
	);
	print "</TD></TR>\n";
	#------------------------------------------------
	# ending day row
	#------------------------------------------------
	print "<TR><TH ALIGN=right BGCOLOR=$bg>End</TH><TD>";
	print $q->popup_menu(
		-name	=> 'eday',
		-values	=> [1..31],
		-default=> $def{eday},
	);
	print $q->popup_menu(
		-name	=> 'emonth',
		-values	=> [1..12],
		-default=> $def{emonth},
		-labels	=> \%MONTH_LABELS,
	);
	print $q->popup_menu(
		-name	=> 'eyear',
		-values	=> [2002..2010],
		-default=> $def{eyear},
	);
	print "</TD></TR>\n";
	#------------------------------------------------
	# what row
	#------------------------------------------------
	print "<TR><TH ALIGN=right BGCOLOR=$bg>What</TH><TD>";
	print $q->popup_menu(
		-name	=> 'type',
		#-values	=> ['Vacation', 'Training', 'Travel', 'CV', 'Other'],
		-values	=> [AbsenceTypes::getTypes()],
		-default=> $def{type},
	);
	print "</TD></TR>\n";
	#------------------------------------------------
	# long-desc row
	#------------------------------------------------
	print "<TR><TH ALIGN=right BGCOLOR=$bg>Description</TH><TD>";
	print $q->textfield(
		-name		=> 'desc',
		-size		=> 50,
		-maxlength	=> 100,
		-default	=> $def{desc},
	);
	print "</TD></TR>\n";
	print "</TABLE>\n";
}

sub showHoliday
{
	my ($q, $rref) = @_;

	print $q->start_html('Absence: Holiday');

	my $href = $rref->[1];
	my ($d, $m, $y, $holiday) = @{$href};
	my $mon = AbsenceDate::monthName($m, '-short');
	my $wd = AbsenceDate::weekDayFromDate($d, $m, $y, '-name');
	print $q->h1("$wd, $d-$mon-$y, is $holiday");
}

sub badClick
{
	my ($q, $more) = @_;

	print $q->start_html('Absence: no-action');
	if ($more eq 'stop-that') {
		print "Don't click there again.<BR>\n";
		return;
	}
	print <<_EOF_;
<P>Instructions:<BR>
<UL>
	<LI>month-days coloured purple are holidays.  You can click
		on them to get the holiday.
	<LI>absences are blocks of red.  Clicking on them allows you to
		display, modify, and delete them.
	<LI>To create a new absence, click on the day in the person-row
		when the absence should start.
</UL>
_EOF_
}
