#!/usr/bin/perl

#=====================================================================
# Welcome to "absence-manage.pl"!
#
# absence-manage.pl is somewhat complicated because it is called at
# two stages (see also the comments at the top of absence-click.pl).
#
# Stage 1:
#
# user clicks on the "manage" URL in the control frame and is presented
# with a number of forms for managing people and groups.
#
# Stage 2:
#
# user clicks on one of the submit buttons belonging to one of the
# forms above and the script is re-entered with some CGI parameters
# set.
#=====================================================================

use CGI;
use FileHandle;

BEGIN {
	if ($ENV{QUERY_STRING} =~ /^instance=(\S+)$/) {
		# an instance has been specified
		$ENV{INSTANCE_NAME} = $1;
	}
}

use AbsenceDB;
use AbsenceConfig;
use AbsenceLog;

#--------------------------------------------------------------------
# global variables
#--------------------------------------------------------------------
$IMAGE_DIR		= AbsenceConfig::fetch('image_dir_abs');
$DATA_DIR_REL	= AbsenceConfig::fetch('data_dir_rel');
$MAIN_SCRIPT	= AbsenceConfig::fetch('main_script');
$DEBUG_OUT		= '/tmp/manage.log';
$VERSION		= '1.1';
$DEBUG			= 0;

$CGI::POST_MAX = 100;
$CGI::DISABLE_UPLOADS = 1;

#--------------------------------------------------------------------
# main()
#--------------------------------------------------------------------
if (exists($ENV{INSTANCE_NAME})) {
	$MAIN_SCRIPT .= "?instance=$ENV{INSTANCE_NAME}";
}

$q = new CGI;
$q->import_names;

# DEBUG
@keywords = $q->keywords;
@names = $q->param;
dbg('Parameters:');
foreach $p (@names) {
	dbg(" [$p] ".$q->param($p));
}

dbg("Keywords:");
foreach $p (@keywords) {
	dbg(" [$p] ");
}


if (defined($Q::action)) {
	if ($Q::action eq 'Add Person') {
		addPerson($q);
	} elsif ($Q::action eq 'Delete Person') {
		deletePerson($q);
	} elsif ($Q::action eq 'Modify Person') {
		modifyPersonForm($q);
	} elsif ($Q::action eq 'Modify Group') {
		modifyGroupForm($q);
	} elsif ($Q::action eq 'Delete Group') {
		deleteGroup($q);
	} elsif ($Q::action eq 'Add Group') {
		addGroup($q);
	} elsif ($Q::action eq 'Submit Modification') {
		modifyPerson($q);
	} elsif ($Q::action eq 'Submit Group Modification') {
		modifyGroup($q);
	}
}

print $q->header(-expires => '-1d');
print $q->start_html(
	-title => 'Absence: Manage People',
	-target => 'display',
);

print $q->h1('Manage People');

print "<P>\n";

#---------------------------------------------------------------
# ADD PERSON
#---------------------------------------------------------------

print $q->h1('Add a person');
print $q->start_form;
print "<TABLE>\n";

#------------------------
# Name
#------------------------
print "<TR><TH ALIGN=right>Name (Last, First)</TH><TD>\n";
print $q->textfield(
	-name		=> 'name',
	-size		=> 20,
	-maxlength	=> 20,
);
print "</TD></TR>\n";

#------------------------
# E-mail
#------------------------
print "<TR><TH ALIGN=right>E-Mail Address</TH><TD>\n";
print $q->textfield(
	-name		=> 'email',
	-size		=> 30,
	-maxlength	=> 50,
);

#------------------------
# groups
#------------------------
print "<TR><TH ALIGN=right>Group</TH><TD>\n";
makeGroupPopup($q);
print "</TD></TR></TABLE>\n";

print $q->submit(action => 'Add Person');
print $q->end_form();

print "<HR>\n";

#---------------------------------------------------------------
# MODIFY PERSON
#---------------------------------------------------------------

print $q->h1('Modify a person');
print $q->start_form;
print "Person";
makePeoplePopup($q);

print $q->submit(action => 'Modify Person');
print $q->end_form();
print "<HR>\n";

#---------------------------------------------------------------
# DELETE PERSON
#---------------------------------------------------------------

print $q->h1('Delete a person');
print $q->start_form;
print "Person";
makePeoplePopup($q);

print $q->submit(action => 'Delete Person');
print $q->end_form();

print "<HR>\n";

#---------------------------------------------------------------
# ADD GROUP
#---------------------------------------------------------------

print $q->h1('Add a Group');
print $q->start_form;
print "<TABLE>\n";
print "<TR><TH ALIGN=right>Group Name</TH><TD>\n";
print $q->textfield(
	-name		=> 'name',
	-size		=> 20,
	-maxlength	=> 20,
);
print "</TD></TR>\n";

## not using password stuff now
##print "<TR><TH ALIGN=right>Password</TH><TD>\n";
##print $q->textfield(
##	-name		=> 'pass',
##	-size		=> 20,
##	-maxlength	=> 20,
##);

print "</TD></TR></TABLE>\n";

print $q->submit(action => 'Add Group');
print $q->end_form();

#---------------------------------------------------------------
# MODIFY GROUP
#---------------------------------------------------------------

print "<HR>\n";

print $q->h1('Modify a group');
print $q->start_form;
print "Group";
makeGroupPopup($q);
print $q->submit(action => 'Modify Group');
print $q->end_form();

print "<HR>\n";

#---------------------------------------------------------------
# DELETE GROUP
#---------------------------------------------------------------

print $q->h1('Delete a group');
print "(this will also delete <EM>all</EM> members of the group)<BR>\n";
print $q->start_form;
print "Group";
makeGroupPopup($q);
print $q->submit(action => 'Delete Group');
print $q->end_form();

print "<HR>\n";

print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
print $q->submit(action => 'Cancel');
print $q->end_form();
print $q->end_html();

exit;


sub makePeoplePopup
{
	my $q = shift;

	my @people = AbsenceDB::getPeople(0);
	my $labref;
	foreach my $person_id (@people) {
		$labref->{$person_id} = AbsenceDB::getPerson($person_id);
	}

	print $q->popup_menu(
		-name   => 'person_id',
		-values => \@people,
		-labels	=> $labref,
	);
}

sub removeMonthImages
{
	my $gid = shift;

	local(*DIR);
	opendir(DIR, $IMAGE_DIR) || die "opendir [$IMAGE_DIR]";
	
	my $it = AbsenceConfig::fetch('image_type');
	my $file;
	while($_ = readdir(DIR)) {
		if (($gid && /^absence-${gid}-.*\.$it$/) ||
			(!$gid && /^absence-.*\.$it$/))
		{
			$file = "$IMAGE_DIR/$_";
			dbg("unlinking [$file]");
			unlink($file) || dbg("failed: $!");
		}
	}
	closedir(DIR);
}

#=================================================================
# add a group
#=================================================================

sub addGroup
{
	my $q = shift;

	dbg("- add group -");

	if (length($Q::name) == 0) {
		dbg("looks bad");
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error adding group',
			-target => 'display',
		);
		print $q->h1("You must supply a name.");
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
		exit;
	}
	my $ret = AbsenceDB::addGroup('new', $Q::name, $Q::pass);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		#print $q->redirect($MAIN_SCRIPT);
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Group Successfully Added',
			-target => '_top',
		);
		print $q->h1("Finished.  Click on button below to reload top page");
		print $q->start_form(-action => $DATA_DIR_REL, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
		exit;
		#removeMonthImages();
	} elsif ($ret eq 'duplicate') {
		dbg("looks bad");
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error adding person',
			-target => 'display',
		);
		print $q->h1("[$Q::name] already exists in system");
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
	}
	exit;
}

sub deleteGroup
{
	my $q = shift;

	dbg("- delete group -");

	my $ret = AbsenceDB::deleteGroup($Q::group_id);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		#print $q->redirect($MAIN_SCRIPT);
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Group Successfully Deleted',
			-target => '_top',
		);
		print $q->h1("Finished.  Click on button below to reload top page");
		print $q->start_form(-action => $DATA_DIR_REL, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
		removeMonthImages($Q::group_id);
		exit;
	} else {
		dbg("looks bad");
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error deleting group',
			-target => 'display',
		);
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
	}
	exit;
}

#=================================================================
# add a person
#=================================================================

sub addPerson
{
	my $q = shift;

	dbg("- add person -");

	if (length($Q::name) == 0) {
		dbg("looks bad");
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error adding person',
			-target => 'display',
		);
		print $q->h1("You must supply a name.");
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
		exit;
	}
	my $ret = AbsenceDB::addPerson('new', $Q::name, $Q::email, $Q::group_id);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		print $q->redirect($MAIN_SCRIPT);
		removeMonthImages($Q::group_id);
	} elsif ($ret eq 'duplicate') {
		dbg("looks bad");
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error adding person',
			-target => 'display',
		);
		print $q->h1("[$Q::name] already exists in system");
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
	}
	exit;
}

sub modifyPerson
{
	my $q = shift;

	dbg("- modify person -");
	if (length($Q::name) == 0) {
		dbg("looks bad");
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error modifying person',
			-target => 'display',
		);
		print $q->h1("You must supply a name.");
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
		exit;
	}
	my $oldgid = (AbsenceDB::getPerson($Q::person_id))[2];
	my $ret = AbsenceDB::addPerson($Q::person_id, $Q::name, $Q::email, $Q::group_id);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		print $q->redirect($MAIN_SCRIPT);
		if ($oldgid != $Q::group_id) {
			dbg("removing all month-images");
			removeMonthImages();
		} else {
			dbg("removing month-images for gid=$Q::group_id");
			removeMonthImages($Q::group_id);
		}
	} else {
		dbg("looks bad");
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error modifying person',
			-target => 'display',
		);
		print $q->h1("'something' went wrong when trying to modify [$Q::name]");
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();

		abslog("error modifying person_id [$Q::person_id] ret = [$ret]");
	}
}

sub modifyPersonForm
{
	my $q = shift;

	my ($name, $email, $gid) = AbsenceDB::getPerson($Q::person_id);
	defined($name) || die "did not get valid name for pid=[$Q::person_id]";

	print $q->header();
	print $q->start_html(
		-title => "Absence: Modify $name",
		-target => 'display',
	);
	print $q->h1("Modify $name");
	print $q->start_form;
	print $q->hidden(-name => 'person_id', -default => $Q::person_id);
	print "<TABLE>\n";

	#---------------------------------------------
	# name
	#---------------------------------------------
	print "<TR><TH ALIGN=right>Name (Last, First)</TH><TD>\n";
	print $q->textfield(
		-name		=> 'name',
		-size		=> 20,
		-maxlength	=> 20,
		-value		=> $name,
	);
	print "</TD></TR>\n";

	#---------------------------------------------
	# E-Mail address
	#---------------------------------------------
	print "<TR><TH ALIGN=right>E-Mail Address</TH><TD>\n";
	print $q->textfield(
		-name		=> 'email',
		-size		=> 30,
		-maxlength	=> 50,
		-value		=> $email,
	);
	print "</TD></TR>\n";

	#---------------------------------------------
	# group
	#---------------------------------------------
	print "<TR><TH ALIGN=right>Group</TH><TD>\n";

	abslog("calling makeGroupPopup with gid=$gid");
	makeGroupPopup($q, $gid);

	print "</TD></TR></TABLE>\n";

	print $q->submit(action => 'Submit Modification');
	print $q->end_form();
	print $q->end_html();
	exit;
}

sub modifyGroup
{
	my $q = shift;

	dbg("- modify group -");
	if (length($Q::name) == 0) {
		dbg("looks bad");
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error modifying group',
			-target => 'display',
		);
		print $q->h1("You must supply a name.");
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
		exit;
	}
	my $ret = AbsenceDB::addGroup($Q::group_id, $Q::name, $Q::pass);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		#print $q->redirect($MAIN_SCRIPT);
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Group Successfully Modified',
			-target => '_top',
		);
		print $q->h1("Finished.  Click on button below to reload top page");
		print $q->start_form(-action => $DATA_DIR_REL, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
		exit;
		#removeMonthImages();
	} else {
		dbg("looks bad");
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error modifying group',
			-target => 'display',
		);
		print $q->h1("'something' went wrong when trying to modify [$Q::name]");
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();

		abslog("error modifying group_id [$Q::group_id] ret = [$ret]");
	}
}

sub modifyGroupForm
{
	my $q = shift;

	my ($name, $pass) = AbsenceDB::getGroupInfo($Q::group_id);

	print $q->header();
	print $q->start_html(
		-title => "Absence: Modify Group $name",
		-target => 'display',
	);
	print $q->h1('Modify a Group');
	print $q->start_form;
	print $q->hidden(-name => 'group_id', -default => $Q::group_id);
	print "<TABLE>\n";
	print "<TR><TH ALIGN=right>Group Name</TH><TD>\n";
	print $q->textfield(
		-name		=> 'name',
		-size		=> 20,
		-maxlength	=> 20,
		-value		=> $name,
	);
	print "</TD></TR>\n";

	## not using password stuff now
	##print "<TR><TH ALIGN=right>Password</TH><TD>\n";
	##print $q->textfield(
	##	-name		=> 'pass',
	##	-size		=> 20,
	##	-maxlength	=> 20,
	##	-value		=> $pass,
	##);

	print "</TD></TR></TABLE>\n";

	print $q->submit(action => 'Submit Group Modification');
	print $q->end_form();
	exit;
}

sub makeGroupPopup
{
	my ($q, $default) = @_;

	my @groups = AbsenceDB::getGroups();
	my $labref;
	foreach my $group_id (@groups) {
		$labref->{$group_id} = AbsenceDB::getGroupInfo($group_id);
	}

	print $q->popup_menu(
		-name   	=> 'group_id',
		-values		=> \@groups,
		-labels		=> $labref,
		-default	=> $default,
	);
}

sub deletePerson
{
	my $q = shift;

	# get gid first
	my $gid = AbsenceDB::getGroup($Q::person_id);

	my $ret = AbsenceDB::deletePerson($Q::person_id);
	if ($ret eq 'ok') {
		print $q->redirect($MAIN_SCRIPT);
		removeMonthImages($gid);
	} elsif ($ret eq 'bad-id') {
		print $q->header();
		print $q->start_html(
			-title => 'Absence: Error deleting person',
			-target => 'display',
		);
		print $q->h1("cannot find this person_id [$Q::person_id] in system");
		print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
		print $q->submit(action => 'Jump back to Calendar');
		print $q->end_form();
		print $q->end_html();
	}
	exit;
}

sub dbg
{
	my $msg = shift;
	$DEBUG || return;
	abslog("DEBUG: absence-manage: $msg");
}

sub old_dbg
{
	$DEBUG || return;

	my $msg = shift;
	my $fh = FileHandle->new(">>$DEBUG_OUT");
	defined($fh) || die "open $DEBUG_OUT for appending";
	print $fh "$msg\n";
	$fh->close;
}

__END__

=head1 NAME

absence-manage.pl - manage people and groups

=head1 DESCRIPTION

C<absence-manage.pl> is responsible for displaying some forms for
the management of people and groups, and handling the processing
after the user clicks on submit in one of the forms.

=cut
